from skbuild import setup

setup(
    name="manifest-in",
    version="1.2.3",
    description="a minimal example package with a MANIFEST.in",
    author='The scikit-build team',
    license="MIT",
    packages=['hello'],
)
